import { CanvasJpPoint } from "./Point";
export declare type CanvasJpColorHsv = {
    __type: "Color";
    h: number;
    s: number;
    v: number;
    a: number;
    hex: () => string;
};
/**
 *
 * @param hue [0,1]
 * @param saturation [0,1]
 * @param value [0,1]
 * @returns
 */
export declare const Color: {
    (hue: number, saturation: number, value: number, alpha?: number): CanvasJpColorHsv;
    /**
     * @param colorA
     * @param colorB
     * @param factor [0,1] 0 = only a, 1 = only b
     * @returns
     */
    mix(colorA: CanvasJpColorHsv, colorB: CanvasJpColorHsv, factor: number): CanvasJpColorHsv;
};
export declare type CanvasJpGradient = {
    __type: "Gradient";
    colors: CanvasJpColorHsv[];
    angle?: number;
};
export declare const Gradient: (colors: CanvasJpColorHsv[], angle?: number | undefined) => CanvasJpGradient;
export declare type CanvasJpRadialGradient = {
    __type: "RadialGradient";
    colors: CanvasJpColorHsv[];
    center: CanvasJpPoint;
    radius: number;
};
export declare const RadialGradient: (colors: CanvasJpColorHsv[], center: CanvasJpPoint, radius: number) => CanvasJpRadialGradient;
export declare const getColorAverage: (color: CanvasJpColorHsv | CanvasJpGradient | CanvasJpRadialGradient) => CanvasJpColorHsv;
/**
 * https://axonflux.com/handy-rgb-to-hsl-and-rgb-to-hsv-color-model-c
 * Converts an HSV color value to RGB. Conversion formula
 * adapted from http://en.wikipedia.org/wiki/HSV_color_space.
 * Assumes h, s, and v are contained in the set [0, 1] and
 * returns r, g, and b in the set [0, 255].
 *
 * @param   Number  hue       The hue
 * @param   Number  saturation       The saturation
 * @param   Number  value       The value
 * @return  Array           The RGB representation
 */
export declare function hsvToRgb(hue: number, saturation: number, value: number): [number, number, number];
/**
 * https://axonflux.com/handy-rgb-to-hsl-and-rgb-to-hsv-color-model-c
 * Converts an RGB color value to HSV. Conversion formula
 * adapted from http://en.wikipedia.org/wiki/HSV_color_space.
 * Assumes r, g, and b are contained in the set [0, 255] and
 * returns h, s, and v in the set [0, 1].
 *
 * @param   Number  red     The red color value [0,255]
 * @param   Number  green   The green color value [0,255]
 * @param   Number  blue    The blue color value [0,255]
 * @return  Array           The HSV representation
 */
export declare function rgbToHsv(red: number, green: number, blue: number): [number, number, number];
export declare function rgbToHex([r, g, b]: [number, number, number]): string;
export declare const isColorClose: (colorsA: CanvasJpColorHsv, colorB: CanvasJpColorHsv, approximation: number) => boolean;
export declare const black: CanvasJpColorHsv;
export declare const white: CanvasJpColorHsv;
export declare const green: CanvasJpColorHsv;
export declare const cyan: CanvasJpColorHsv;
export declare const red: CanvasJpColorHsv;
export declare const colorFromHex: (hex: string) => CanvasJpColorHsv;
